//  MatMult.java  -  Demonstrate 2-D arrays and matrix multiplication

public class MatMult {

    public static int[][] matMult(int[][] x, int[][] y) {
        int[][] z = null;
        if (x[0].length == y.length) {
            z = new int[x.length][y[0].length];
            for (int i = 0; i < x.length; i++) {
                for (int j = 0; j < y[0].length; j++) {
                    int sum = 0;
                    for (int k = 0; k < x[0].length; k++)
                        sum += x[i][k] * y[k][j];
                    z[i][j] = sum;
                }
            }
        } else
            System.err.println("ERROR:  matMult -- incompatible matrix sizes");
        return z;
    }

    public static void display(String name, int[][] mat) {
        System.out.println("Matrix: " + name);
        for (int i = 0; i < mat.length; i++) {
            for (int j = 0; j < mat[i].length; j++)
                System.out.print(" " + mat[i][j]);
            System.out.println();
        }
        System.out.println();
    }

    public static void main(String[] args) {
        System.out.println("Demonstate MatMult - Matrix Multiplication");
        System.out.println();
        int[][] a = { { 1, 2 },
                      { 3, 3 },
                      { 2, 1 } };
        int[][] b = { { 1, 0, 1 },
                      { 0, 1, 0 } };
        int[][] ab, ba;
        display("a", a);
        display("b", b);
        ab = matMult(a, b);
        display("ab = a x b", ab);
        ba = matMult(b, a);
        display("ba = b x a", ba);
        ab = matMult(ab, ab);
        display("ab * ab", ab);
        ba = matMult(ba, ba); 
        display("ba * ba", ba);
    }

}

/* ===== execution output =====

Demonstate MatMult - Matrix Multiplication

Matrix: a
 1 2
 3 3
 2 1

Matrix: b
 1 0 1
 0 1 0

Matrix: ab = a x b
 1 2 1
 3 3 3
 2 1 2

Matrix: ba = b x a
 3 3
 3 3

Matrix: ab * ab
 9 9 9
 18 18 18
 9 9 9

Matrix: ba * ba
 18 18
 18 18

Exit code: 0

===== end output ===== */

